//
//  BloglinesMenuController.m
//  BloglinesMenu
//
// Copyright 2005 Codepope/Runstate
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.


#import "BloglinesMenuController.h"
#import "PreferencesController.h"

#import <Growl/GrowlApplicationBridge.h>
#import <Growl/GrowlDefines.h>

#import "Socket.h"

#define GROWL_NOTIFICATION_DEFAULT @"NotificationDefault"

static NSString *appName = @"BloglinesMenu";

@implementation BloglinesMenuController

-(void)awakeFromNib
{
	
	statusItem=[[[NSStatusBar systemStatusBar]statusItemWithLength:NSVariableStatusItemLength] retain];
	
	pollrate=[[[NSUserDefaults standardUserDefaults] stringForKey:CDCPollRate] intValue];
	
	mainTimer=[[NSTimer scheduledTimerWithTimeInterval:(pollrate)
												target:self
											  selector:@selector(timer:)
											  userInfo:nil
											   repeats:YES] retain];
	
	[statusItem setHighlightMode:YES];
	[statusItem setTitle:@"*"];
	[statusItem setToolTip:NSLocalizedString(@"StartingBloglinesMenu",@"StartingBloglinesMenu")];
	[statusItem setMenu:theMenu];
	[statusItem setEnabled:YES];
	
	
	
	
	[mainTimer fire];
}

+(void)initialize
{
	NSMutableDictionary *defaultValues=[NSMutableDictionary dictionary];
	[defaultValues setObject:@"" forKey: CDCEmailAddressKey];
	[defaultValues setObject:@"30" forKey:CDCPollRate];
	[defaultValues setObject:@"1" forKey:CDCNotifyGrowl];
	[defaultValues setObject:@"0" forKey:CDCNotifyBeep];
	[defaultValues setObject:@"10" forKey:CDCSnoozeTime];
	[defaultValues setObject:@"0" forKey:CDCAutoSnooze];
	[defaultValues setObject:@"0" forKey:CDCNixonMode];
	[defaultValues setObject:@"0" forKey:CDCStartAtLogin];
	[[NSUserDefaults standardUserDefaults] registerDefaults:defaultValues];
	
}



-(id)init
{
	if(self=[super init])
	{
		// For listening to changes in preferences....
		NSNotificationCenter *nc;
		nc=[NSNotificationCenter defaultCenter];
		[nc addObserver:self selector:@selector(handlePrefsChange:) name:@"CDCPrefsChanged" object:nil];
		
		oldunread=-1;
		snoozing=false;
		
		if([GrowlApplicationBridge isGrowlInstalled])
		{
				[GrowlApplicationBridge setGrowlDelegate:self];
			
			
		}
	}
	return self;
}

-(void)dealloc
{   
	NSNotificationCenter *nc;
    nc=[NSNotificationCenter defaultCenter];
    [nc removeObserver:self];
	
	[iconData release];
	[statusItem release];
	[mainTimer invalidate];
	[mainTimer release];
	
	[preferencesController release];
	
	[super dealloc];
}

-(void)notifyUnread:(int)unread
{
	NSString *growl=[[NSUserDefaults standardUserDefaults] stringForKey:CDCNotifyGrowl];
	NSString *beep=[[NSUserDefaults standardUserDefaults] stringForKey:CDCNotifyBeep];
	
	if([growl intValue]==1)
	{
		NSDictionary *noteDict;
		NSImage *icon=[NSImage imageNamed:appName];
		iconData=[[icon TIFFRepresentation] retain];
		
		NSString *descstring=unread==1?
			[NSString stringWithFormat:NSLocalizedString(@"GrowlBloglinesNewUnreadMessage",@"GrowlBloglinesNewUnreadMessage"),unread]:
			[NSString stringWithFormat:NSLocalizedString(@"GrowlBloglinesNewUnreadMessages",@"GrowlBloglinesNewUnreadMessage"),unread];
		
		noteDict = [NSDictionary dictionaryWithObjectsAndKeys:
			BLOGLINESMENU_MOREUNREAD, GROWL_NOTIFICATION_NAME,
			appName, GROWL_APP_NAME,
			NSLocalizedString(@"GrowlBloglinesNewUnread",@"GrowlBloglinesNewUnread"), GROWL_NOTIFICATION_TITLE,
			descstring, GROWL_NOTIFICATION_DESCRIPTION,
			[icon TIFFRepresentation], GROWL_NOTIFICATION_ICON, 
			nil];
		
//		[[NSDistributedNotificationCenter defaultCenter] postNotificationName:GROWL_NOTIFICATION
//																	   object:nil userInfo:noteDict];
	
		[GrowlApplicationBridge notifyWithDictionary:noteDict];
	}
	
	if([beep intValue]==1)
	{
		NSBeep();
	}
}

- (NSDictionary *) registrationDictionaryForGrowl {  
 
	NSArray *notifications=[NSArray arrayWithObject:BLOGLINESMENU_MOREUNREAD ];

 	 return [NSDictionary dictionaryWithObjectsAndKeys:notifications, GROWL_NOTIFICATIONS_ALL, notifications, GROWL_NOTIFICATIONS_DEFAULT, nil];  
	}  
	
	
-(void)timer:(NSTimer *)timer
{
	NSString *bloglinesurlstring;
	NSURL *bloglinesurl;
	NSString *result;
	NSString *emailaddress;
	NSArray *resultarray;
	NSString *resultvalue;
	NSString *httpRequest;
	NSMutableData *response;
	Socket *socket;
	  
	emailaddress=[[NSUserDefaults standardUserDefaults] stringForKey:CDCEmailAddressKey];
	
	if([emailaddress length]==0) 
	{
		[statusItem setTitle:@"!"];
		[statusItem setToolTip:NSLocalizedString(@"CheckEmailAddress",@"CheckEmailAddress")];
		[self openPreferences:self];
		return;
	}
	

	httpRequest = [NSString stringWithFormat:@"GET %@ HTTP/1.0\r\n\r\n", [NSString stringWithFormat:@"/update?user=%@&ver=1",emailaddress]];
	
	// Create socket, connect, and send request
	
	socket = [Socket socket];
	
	[socket connectToHostName:@"rpc.bloglines.com" port:80];
	
	[socket writeString:httpRequest];

	response = [[[NSMutableData alloc] init] autorelease];
	
	while ( [socket readData:response] )
	{
		// Read until other side disconnects
	}
	
	result = [[[NSString alloc] initWithData:response 
											encoding:[NSString defaultCStringEncoding]] autorelease];

	if([result length]!=0)
	{
		resultarray=[result componentsSeparatedByString:@"|"];
		
		resultvalue=[resultarray objectAtIndex:1];
		
		if([resultvalue isEqualToString:@"-1"])
		{
			[statusItem setTitle:@"!"];
			[statusItem setToolTip:NSLocalizedString(@"CheckEmailAddress",@"CheckEmailAddress")];
		}
		else
		{
			int unread;
			
			unread=[resultvalue intValue];
			
			if(oldunread==-1)
			{
				if(unread>0)
				{
					[self notifyUnread:unread];
				}
				
				oldunread=unread;
			}
			else
			{
				if(unread>oldunread)
				{
					[self notifyUnread:unread];
				}
				
				oldunread=unread;
			}
			
			NSString *descstring=unread==1?
				[NSString stringWithFormat:NSLocalizedString(@"TooltipUnreadMessage",@"TooltipUnreadMessage"),unread]:
				[NSString stringWithFormat:NSLocalizedString(@"TooltipUnreadMessages",@"TooltipUnreadMessage"),unread];
			
			[statusItem setTitle:[resultarray objectAtIndex:1]];
			[statusItem setToolTip:descstring];
		}
	}
	else
	{
		[statusItem setTitle:@"T"];
		[statusItem setToolTip:NSLocalizedString(@"ErrorWhileChecking",@"ErrorWhileChecking")];
	}
}

-(IBAction)snoozeNow:(id)sender
{
	[mainTimer invalidate];
	[mainTimer release];

	if([[[NSUserDefaults standardUserDefaults] stringForKey:CDCNixonMode] intValue]!=0)
	{
		[statusItem setTitle:@"0"];
	}
	else
	{
		[statusItem setTitle:@"Z"];
	}
	
	[statusItem setToolTip:@"Snoozing"];
	
	snoozetime=[[[NSUserDefaults standardUserDefaults] stringForKey:CDCSnoozeTime] intValue];
	
	snoozing=true;
	oldunread=-1;
	
	mainTimer=[[NSTimer scheduledTimerWithTimeInterval:(snoozetime*60)
												target:self
											  selector:@selector(snoozeOver:)
											  userInfo:nil
											   repeats:NO] retain];
	
}

-(void)snoozeOver:(NSTimer *)timer
{
	[mainTimer invalidate];
	[mainTimer release];
	
	snoozing=false;
	
	mainTimer=[[NSTimer scheduledTimerWithTimeInterval:(pollrate)
												target:self
											  selector:@selector(timer:)
											  userInfo:nil
											   repeats:YES] retain];
	
	[mainTimer fire];
}

-(IBAction)checkNow:(id)sender
{
	if(snoozing)
	{
		[self snoozeOver:nil];
	}
	else
	{
		[mainTimer fire];
	}
}

-(void)handlePrefsChange:(NSNotification *)aNotification
{
	[mainTimer invalidate];
	[mainTimer release];
	
	pollrate=[[[NSUserDefaults standardUserDefaults] stringForKey:CDCPollRate] intValue];
	snoozetime=[[[NSUserDefaults standardUserDefaults] stringForKey:CDCSnoozeTime] intValue];

	if(!snoozing)
	{
				
		mainTimer=[[NSTimer scheduledTimerWithTimeInterval:(pollrate)
													target:self
												  selector:@selector(timer:)
												  userInfo:nil
												   repeats:YES] retain];
		
		[mainTimer fire];
	}
	else
	{
		
		mainTimer=[[NSTimer scheduledTimerWithTimeInterval:(snoozetime*60)
													target:self
												  selector:@selector(snoozeOver:)
												  userInfo:nil
												   repeats:NO] retain];
	}
	
}

-(IBAction)openBloglines:(id)sender
{
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://www.bloglines.com/myblogs"]];

	if(!snoozing && [[[NSUserDefaults standardUserDefaults] stringForKey:CDCAutoSnooze] intValue]!=0)
	{
		[self snoozeNow:nil];
	}
	
}

-(IBAction)openPreferences:(id)sender
{
	if(!preferencesController)
		preferencesController=[[PreferencesController alloc] init];
	
	[NSApp activateIgnoringOtherApps:YES];
	[[preferencesController window] makeKeyAndOrderFront:nil];
}

-(IBAction)openAbout:(id)sender
{
	[NSApp orderFrontStandardAboutPanel:nil];
}



@end
